﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;
using HeuristicLab.Problems.DataAnalysis;
using HeuristicLab.Problems.Instances.DataAnalysis.Regression.Matlab.Api;
using HeuristicLab.Problems.Instances.DataAnalysis.Regression.Matlab.Api.Types;

namespace HeuristicLab.Problems.Instances.DataAnalysis.Views.Regression.Matlab {
  public partial class RegressionMatlabImportDialog {
    private class MLTimeseriesLoader : MLVariablesLoader {
      public MLTimeseriesLoader(RegressionMatlabImportDialog importDialog, CheckedListBox variablesListBox) 
        : base(importDialog, variablesListBox, MLDatatype.Timeseries) {
      }      

      public override Dataset GetPreviewDataset(IMatlabConnector mlConnector, CancellationToken token) {
        var timeseries = new List<MLTimeseries>();
        foreach (MLVariableInfo item in _variablesListBox.CheckedItems.Cast<MLVariableInfo>().ToList()) {
          if (token.IsCancellationRequested) {
            return null;
          }

          if (item.Datatype == _mlDatatype) {
            var val = mlConnector.GetTimeseries(item.VarName);
            if (val != null) {
              timeseries.Add(val);
            }
          }
        }

        var merged = new MLTimeseries(timeseries, token);
        if (token.IsCancellationRequested) {
          return null;
        }
        return new Dataset(GetVariableNames(merged), GetValuesForDataset(merged));
      }      

      /// <summary>
      /// Returns a list with all selected variable names for the preview dataset.
      /// </summary>
      /// <param name="timeseries"></param>
      /// <returns></returns>
      private IEnumerable<string> GetVariableNames(MLTimeseries timeseries) {
        IList<string> variableNamesWithType = timeseries.DataHeader.ToList();
        variableNamesWithType.Insert(0, "time");
        return variableNamesWithType;
      }

      /// <summary>
      /// Returns a two dimensional double array which contains the values including the timestamps of the given timeseries.
      /// Array[value, variable] 
      /// If the amount of the data is too big, a OutOfMemoryException exception is being thrown. 
      /// This exception is being cought and a message will be shown at the error message box.
      /// </summary>
      /// <param name="timeseries"></param>
      /// <returns></returns>
      private double[,] GetValuesForDataset(MLTimeseries timeseries) {
        double[,] datasetValues = new double[timeseries.Count, timeseries.DataHeader.Length + 1];

        int i = 0;
        foreach (var item in timeseries.Data) {
          datasetValues[i, 0] = item.Key;
          for (int j = 1; j < datasetValues.GetLength(1); j++) {
            datasetValues[i, j] = item.Value[j - 1];
          }
          i++;
        }
        return datasetValues;
      }
    }
  }
}
